/*
 * ARM64 specific ACPICA environments and implementation
 *
 * Copyright (C) 2014, Linaro Ltd.
 *   Author: Hanjun Guo <hanjun.guo@linaro.org>
 *   Author: Graeme Gregory <graeme.gregory@linaro.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef _ASM_ACENV_H
#define _ASM_ACENV_H

/* It is required unconditionally by ACPI core, update it when needed. */

#endif /* _ASM_ACENV_H */
